package sysbench;

import org.apache.commons.cli.BasicParser;
import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;

public final class Config {
	public static final String JDBC_URL = "jdbcUrl";
	public static final String USE_CONNECTION_POOL = "useConnectionPool";
	public static final String USE_PREPARED_STATEMENT_POOL = "usePreparedStatementPool";
	public static final String USE_PREPARED_STATEMENT = "usePreparedStatement";
	public static final String WARMUP_TIME = "warmupTime";
	public static final String MEASUREMENT_TIME = "measurementTime";
	public static final String NUM_WORKERS = "numWorkers";
	public static final String OLTP_TABLE_SIZE = "oltpTableSize";
	public static final String OLTP_RANGE_SIZE = "oltpRangeSize";

	private static final String SEPARATOR = System.getProperty("line.separator");

	private final String jdbcUser = "sbtest";
	private final String jdbcPass = "sbtest";

	private String jdbcUrl = "";
	private boolean useConnectionPool = true;
	private boolean usePreparedStatementPool = true;
	private boolean usePreparedStatement = true;
	private int warmupTime = 60;
	private int measurementTime = 180;
	private int numWorkers = 16;
	private int oltpTableSize = 10000;
	private int oltpRangeSize = 100;

	public Config(String[] args) throws ParseException {
		Options options = new Options();
		CommandLineParser parser = new BasicParser();
		CommandLine cl = null;

		Config.buildOptions(options);
		cl = parser.parse(options, args, true);
		loadOptions(cl);
	}

	private static void buildOptions(Options options) {
		options.addOption(JDBC_URL, true, "");
		options.addOption(USE_CONNECTION_POOL, true, "");
		options.addOption(USE_PREPARED_STATEMENT_POOL, true, "");
		options.addOption(USE_PREPARED_STATEMENT, true, "");
		options.addOption(WARMUP_TIME, true, "");
		options.addOption(MEASUREMENT_TIME, true, "");
		options.addOption(NUM_WORKERS, true, "");
		options.addOption(OLTP_TABLE_SIZE, true, "");
		options.addOption(OLTP_RANGE_SIZE, true, "");
	}

	public String getConfigString() {
		StringBuilder configString = new StringBuilder();

		configString.append(JDBC_URL + " : " + jdbcUrl + SEPARATOR);
		configString.append(USE_CONNECTION_POOL + " : " + useConnectionPool + SEPARATOR);
		configString.append(USE_PREPARED_STATEMENT_POOL + " : " + usePreparedStatementPool
				+ SEPARATOR);
		configString.append(USE_PREPARED_STATEMENT + " : " + usePreparedStatement + SEPARATOR);
		configString.append(WARMUP_TIME + " : " + warmupTime + SEPARATOR);
		configString.append(MEASUREMENT_TIME + " : " + measurementTime + SEPARATOR);
		configString.append(NUM_WORKERS + " : " + numWorkers + SEPARATOR);
		configString.append(OLTP_TABLE_SIZE + " : " + oltpTableSize + SEPARATOR);
		configString.append(OLTP_RANGE_SIZE + " : " + oltpRangeSize + SEPARATOR);

		return configString.toString();
	}

	public String getJdbcUrl() {
		return jdbcUrl;
	}

	public String getJdbcUser() {
		return jdbcUser;
	}

	public String getJdbcPass() {
		return jdbcPass;
	}

	public boolean useConnectionPool() {
		return useConnectionPool;
	}

	public boolean usePreparedStatementPool() {
		return usePreparedStatementPool;
	}

	public boolean usePreparedStatement() {
		return usePreparedStatement;
	}

	public int getWarmupTime() {
		return warmupTime;
	}

	public int getMeasurementTime() {
		return measurementTime;
	}

	public int getNumWorkers() {
		return numWorkers;
	}

	public int getOltpTableSize() {
		return oltpTableSize;
	}

	public int getOltpRangeSize() {
		return oltpRangeSize;
	}

	private void loadOptions(CommandLine cl) {
		if (cl.hasOption(JDBC_URL)) {
			this.jdbcUrl = cl.getOptionValue(JDBC_URL);
		}

		if (cl.hasOption(USE_CONNECTION_POOL)) {
			this.useConnectionPool = Boolean.valueOf(cl.getOptionValue(USE_CONNECTION_POOL));
		}

		if (cl.hasOption(USE_PREPARED_STATEMENT_POOL)) {
			this.usePreparedStatementPool = Boolean.valueOf(cl
					.getOptionValue(USE_PREPARED_STATEMENT_POOL));
		}

		if (cl.hasOption(USE_PREPARED_STATEMENT)) {
			this.usePreparedStatement = Boolean.valueOf(cl.getOptionValue(USE_PREPARED_STATEMENT));
		}

		if (cl.hasOption(WARMUP_TIME)) {
			this.warmupTime = Integer.parseInt(cl.getOptionValue(WARMUP_TIME));
		}

		if (cl.hasOption(MEASUREMENT_TIME)) {
			this.measurementTime = Integer.parseInt(cl.getOptionValue(MEASUREMENT_TIME));
		}

		if (cl.hasOption(NUM_WORKERS)) {
			this.numWorkers = Integer.parseInt(cl.getOptionValue(NUM_WORKERS));
		}

		if (cl.hasOption(OLTP_TABLE_SIZE)) {
			this.oltpTableSize = Integer.parseInt(cl.getOptionValue(OLTP_TABLE_SIZE));
		}

		if (cl.hasOption(OLTP_RANGE_SIZE)) {
			this.oltpRangeSize = Integer.parseInt(cl.getOptionValue(OLTP_RANGE_SIZE));
		}
	}
}
