package sysbench;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.Random;

public abstract class Worker implements Runnable {
	protected final int oltpTableSize;
	protected final int oltpRangeSize;
	private final Manager manager;
	private final int workerId;
	private final Random random;
	private volatile Thread thread;

	public Worker(Manager manager, int workerId, int oltpTableSize, int oltpRangeSize) {
		this.manager = manager;
		this.workerId = workerId;
		this.random = new Random();
		this.oltpTableSize = oltpTableSize;
		this.oltpRangeSize = oltpRangeSize;
	}

	public void start() {
		if (thread == null) {
			this.thread = new Thread(this, "worker" + workerId);
			thread.start();
		}
	}

	public void run() {
		Connection connection = null;

		try {
			while (manager.getStatus()) {
				connection = manager.getConnection();
				executeTransaction(connection);
				connection.close();
				manager.incrementTransactionCount();
			}

		} catch (Exception e) {
			manager.putException(workerId, e);

		} finally {
			if (connection != null) {
				try {
					connection.close();
				} catch (SQLException e) {
				}
			}
		}
	}

	public Thread getThread() {
		return thread;
	}

	protected abstract void executeTransaction(Connection connection) throws SQLException;

	protected int getRandomId() {
		return 1 + random.nextInt(1073741822) % oltpTableSize;
	}

	protected int getRangeRandomId() {
		int id = getRandomId();

		if (id + oltpRangeSize > oltpTableSize) {
			id = oltpTableSize - oltpRangeSize;
		}

		if (id < 1) {
			id = 1;
		}

		return id;
	}
}
